@@ -7,7 +7,10 @@
 static const R_CallMethodDef callMethods[] = {
     CALLDEF(R_ExpectationCovarianceStatistic, 7),
     CALLDEF(R_PermutedLinearStatistic, 6),
+    CALLDEF(R_quadform, 3),
     CALLDEF(R_kronecker, 2),
+    CALLDEF(R_MPinv_sym, 3),
+    CALLDEF(R_unpack_sym, 3),
     CALLDEF(R_maxstattrafo, 2),
     CALLDEF(R_outersum, 2),
     CALLDEF(R_cpermdist2, 5),
